from django.contrib.auth.models import User
import factory

from referral.models import Campaign, Referrer, UserReferrer

class UserFactory(factory.Factory):
    FACTORY_FOR = User

class CampaignFactory(factory.Factory):
    FACTORY_FOR = Campaign

    name = factory.Sequence(lambda n: "Test Campaign %s" % n)
    description = "Some long test description"

class ReferrerFactory(factory.Factory):
    FACTORY_FOR = Referrer

    name = factory.Sequence(lambda n: "Test Referrer %s" % n)
    description = "Some long test description"

class UserReferrerFactory(factory.Factory):
    FACTORY_FOR = UserReferrer

    user = factory.LazyAttribute(lambda a: UserFactory())
    referrer = factory.LazyAttribute(lambda a: ReferrerFactory())




from django.contrib import admin

from models import Campaign, Referrer, UserReferrer

class ReferrerInine(admin.TabularInline):
    model = Referrer
    extra = 0

class CampaignAdmin(admin.ModelAdmin):
    list_display = ('name', 'count_users')
    inlines = (ReferrerInine, )

class ReferrerAdmin(admin.ModelAdmin):
    list_display = ('name', 'campaign', 'creation_date', 'count_users')

class UserReferrerAdmin(admin.ModelAdmin):
    list_display = ('user', 'referrer')
    raw_id_fields = ('user',)

admin.site.register(Campaign, CampaignAdmin)
admin.site.register(Referrer, ReferrerAdmin)
admin.site.register(UserReferrer, UserReferrerAdmin)




from django.test import TestCase
from django.http import HttpRequest

from referral.middleware import ReferrerMiddleware
from referral.models import Referrer 
from referral import settings


class ReferrerMiddlewareTest(TestCase):
    def setUp(self):
        self.ref_middleware = ReferrerMiddleware()
        self.request = HttpRequest()
        self.request.session = {}

    def test_process_request_no_ref(self):
        self.ref_middleware.process_request(self.request)
        self.assertEqual(Referrer.objects.all().count(), 0)

    def test_process_request_new_ref(self):
        self.request.GET[settings.GET_PARAMETER] = "new_ref"
        self.ref_middleware.process_request(self.request)
        self.assertEqual(Referrer.objects.all()[0].name, "new_ref")




from django.conf import settings

# Name of the referrer's get parameter name
GET_PARAMETER = getattr(settings, 'REFERRAL_GET_PARAMETER', 'ref')

# Name of the session variable storing the detected referrer
SESSION_KEY = getattr(settings, 'REFERRAL_SESSION_KEY', 'referrer')

# Should unknown referrers be auto created? Default: True
AUTO_CREATE = getattr(settings, 'REFERRAL_AUTO_CREATE', True)

# If this is set auto created referrers will be associated to a campaign that defines a matching pattern. Default: True
AUTO_ASSOCIATE = getattr(settings, 'REFERRAL_AUTO_ASSOCIATE', True)




from django.http import HttpRequest
from django.test import TestCase

from referral.models import UserReferrer
from referral import settings
from referral.tests.factories import CampaignFactory, ReferrerFactory, UserReferrerFactory, UserFactory


class CampaignTestCase(TestCase):
    def test_model(self):
        obj = CampaignFactory()
        self.assertTrue(obj.pk)

    def test_count_users(self):
        obj = CampaignFactory()
        self.assertEqual(obj.count_users(),0)
        ReferrerFactory(campaign=obj)
        self.assertEqual(obj.count_users(),0)
        ref = ReferrerFactory(campaign=obj)
        UserReferrerFactory(referrer=ref)
        self.assertEqual(obj.count_users(),1)

class ReferrerTestCase(TestCase):
    def test_model(self):
        obj = ReferrerFactory()
        self.assertTrue(obj.pk)

    def test_match_campaign(self):
        obj = ReferrerFactory()
        obj.match_campaign()
        CampaignFactory()
        self.assertIsNone(obj.campaign)
        campaign = CampaignFactory(pattern="Test Referrer")
        obj.match_campaign()
        self.assertEqual(obj.campaign, campaign)

class UserReferrerTestCase(TestCase):
    def test_model(self):
        obj = UserReferrerFactory()
        self.assertTrue(obj.pk)

    def test_manager_apply_referrer_no_ref(self):
        user = UserFactory()
        request = HttpRequest()
        request.session = {}
        UserReferrer.objects.apply_referrer(user, request)
        try:
            user.user_referrer
        except UserReferrer.DoesNotExist:
            pass
        else:
            assert False, "Referrer should not exist!"

    def test_manager_apply_referrer(self):
        referrer = ReferrerFactory()
        user = UserFactory()
        request = HttpRequest()
        request.session = {settings.SESSION_KEY: referrer}
        UserReferrer.objects.apply_referrer(user, request)
        self.assertEqual(user.user_referrer.referrer, referrer)




from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _

import settings


class Campaign(models.Model):
    name = models.CharField(_("Name"), max_length=255, unique=True)
    description = models.TextField(_("Description"), blank=True, null=True)
    pattern = models.CharField(_("Referrer pattern"), blank=True, max_length=255,
            help_text="All auto created referrers containing this pattern will be associated with this campaign")

    class Meta:
        ordering = ['name']
        verbose_name = _("Campaign")
        verbose_name_plural = _("Campaigns")

    def __unicode__(self):
        return self.name

    def count_users(self):
        count = 0
        for referrer in self.referrers.all():
            count += referrer.count_users()
        return count
    count_users.short_description = _("User count")

class Referrer(models.Model):
    name = models.CharField(_("Name"), max_length=255, unique=True)
    description = models.TextField(_("Description"), blank=True, null=True)
    creation_date = models.DateTimeField(_("Creation date"), auto_now_add=True)
    campaign = models.ForeignKey(Campaign, verbose_name=_("Campaign"), related_name='referrers', blank=True, null=True)

    class Meta:
        ordering = ['name']
        verbose_name = _("Referrer")
        verbose_name_plural = _("Referrers")

    def __unicode__(self):
        return self.name

    def count_users(self):
        return self.users.count()
    count_users.short_description = _("User count")

    def match_campaign(self):
        for campaign in Campaign.objects.exclude(pattern=""):
            if campaign.pattern in self.name:
                self.campaign = campaign
                self.save()
                break

class UserReferrerManager(models.Manager):
    def apply_referrer(self, user, request):
        try:
            referrer = request.session.pop(settings.SESSION_KEY)
        except KeyError:
            pass
        else:
            user_referrer = UserReferrer(user=user, referrer=referrer)
            user_referrer.save()

class UserReferrer(models.Model):
    user = models.OneToOneField(User, verbose_name=_("User"), related_name='user_referrer')
    referrer = models.ForeignKey(Referrer, verbose_name=_("Referrer"), related_name='users')

    objects = UserReferrerManager()

    class Meta:
        ordering = ['referrer__name']
        verbose_name = _("User Referrer")
        verbose_name_plural = _("User Referrers")

    def __unicode__(self):
        return "%s -> %s" % (self.user.username, self.referrer.name)




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Campaign'
        db.create_table('referral_campaign', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
        ))
        db.send_create_signal('referral', ['Campaign'])

        # Adding model 'Referrer'
        db.create_table('referral_referrer', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('creation_date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('campaign', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['referral.Campaign'], null=True, blank=True)),
        ))
        db.send_create_signal('referral', ['Referrer'])

        # Adding model 'UserReferrer'
        db.create_table('referral_userreferrer', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.OneToOneField')(related_name='referrer', unique=True, to=orm['auth.User'])),
            ('referrer', self.gf('django.db.models.fields.related.ForeignKey')(related_name='users', to=orm['referral.Referrer'])),
        ))
        db.send_create_signal('referral', ['UserReferrer'])


    def backwards(self, orm):
        # Deleting model 'Campaign'
        db.delete_table('referral_campaign')

        # Deleting model 'Referrer'
        db.delete_table('referral_referrer')

        # Deleting model 'UserReferrer'
        db.delete_table('referral_userreferrer')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'referral.campaign': {
            'Meta': {'ordering': "['name']", 'object_name': 'Campaign'},
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'referral.referrer': {
            'Meta': {'ordering': "['name']", 'object_name': 'Referrer'},
            'campaign': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['referral.Campaign']", 'null': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'})
        },
        'referral.userreferrer': {
            'Meta': {'ordering': "['referrer__name']", 'object_name': 'UserReferrer'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'referrer': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'users'", 'to': "orm['referral.Referrer']"}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'related_name': "'referrer'", 'unique': 'True', 'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['referral']


import settings
from models import Referrer


class ReferrerMiddleware():
    def process_request(self, request):
        if settings.GET_PARAMETER in request.GET:
            referrer = None
            referrer_name = request.GET[settings.GET_PARAMETER]
            try:
                referrer = Referrer.objects.get(name=referrer_name)
            except Referrer.DoesNotExist:
                if settings.AUTO_CREATE:
                    referrer = Referrer(name=referrer_name)
                    referrer.save()
                if settings.AUTO_ASSOCIATE:
                    referrer.match_campaign()
            finally:
                if not referrer is None:
                    request.session[settings.SESSION_KEY] = referrer

